==> emod_api-1.8.2-py3-none-any.whl: emod_api/interventions/utils.py <==
from emod_api import schema_to_class as s2c
import json

def do_nodes( schema_path, node_ids ):
    """
        Create and return a NodeSetConfig based on node_ids list.
    """
    if node_ids and len(node_ids) > 0:
        nodelist = s2c.get_class_with_defaults( "NodeSetNodeList", schema_path )
        nodelist.Node_List = node_ids
    else:
        nodelist = s2c.get_class_with_defaults( "NodeSetAll", schema_path )
    return nodelist


def get_waning_from_params( schema_path, initial=1.0, box_duration=365, decay_rate=0 ):
    """
        Get well configured waning structure. Default is 1-year full efficacy box.
        Note that an infinite decay rate (0 or even -1) is same as Box.
        Note that an infinite box duration (-1) is same as constant.
        Note that a zero box duration is same as Exponential.

    """
    waning = None
    if box_duration == -1:
        waning = s2c.get_class_with_defaults( "WaningEffectConstant", schema_path )
    else:
        waning = s2c.get_class_with_defaults( "WaningEffectBoxExponential", schema_path )
        waning.Box_Duration = box_duration
        if decay_rate > 0: # -1 really
            waning.Decay_Time_Constant = 1.0/decay_rate
        else:
            waning.Decay_Time_Constant = 0

    waning.Initial_Effect = initial
    return waning


def _convert_prs( kvps ):
    """
    This function is not intended to be called directly by users but rather by emod_api.interventions.common
    API functions. It is intended to allow users to target interventions to individuals with specific 
    individual properties without having to worry about the particular data structures that the DTK uses.
    The permissible structures do need to be documented in the relevant API functions.
    kvps needs to be one of:
    - "key:value"
    - "key=value"
    - "key1:value1,key2:value2"
    - "key1=value1,key2=value2"
    - { "key": "value" }
    - { "key1": "value1", "key2": "value2" }
    - [  { "key1": "value1", "key2": "value2" }, { "key3": "value3", "key4": "value4" } ]
    Note that the key-value-pairs _values_ are not currently validated against the selected demographics file.
    In the current implementation, the return value is prepped as a value for Property_Restrictions_Within_Nodes
    """

    ret_kvps = []
    if kvps is None:
        ret_kvps = list() # just happen to know default is empty list, not getting from schema
    elif type(kvps) is str and len(kvps)>0:
        prs_as_dict = dict()
        entries = kvps.split(',')
        for entry in entries:
            if ":" in kvps:
                key, value = entry.split(':')
                prs_as_dict[key] = value
            elif "=" in kvps:
                key, value = entry.split('=')
                prs_as_dict[key] = value
            else:
                print( f"ERROR: malformed property targets: {kvps}." )
        ret_kvps = [ prs_as_dict ]
    elif type(kvps) is dict:
        ret_kvps = [kvps]
    elif type(kvps) is list:
        if len(kvps)>0:
            if type(kvps[0]) is dict:
                ret_kvps = kvps # could check the contents of the list...
            else:
                print( f"ERROR: malformed property targets: {kvps}." )
    else:
        print( f"ERROR: malformed property targets: {kvps}." )
    return ret_kvps

==> emod_api-1.13.2-py3-none-any.whl: emod_api/interventions/utils.py <==
from emod_api import schema_to_class as s2c
import json

def do_nodes( schema_path, node_ids ):
    """
        Create and return a NodeSetConfig based on node_ids list.
    """
    if node_ids and len(node_ids) > 0:
        nodelist = s2c.get_class_with_defaults( "NodeSetNodeList", schema_path )
        nodelist.Node_List = node_ids
    else:
        nodelist = s2c.get_class_with_defaults( "NodeSetAll", schema_path )
    return nodelist


def get_waning_from_params( schema_path, initial=1.0, box_duration=365, decay_rate=0 ):
    """
        Get well configured waning structure. Default is 1-year full efficacy box.
        Note that an infinite decay rate (0 or even -1) is same as Box.
        Note that an infinite box duration (-1) is same as constant.
        Note that a zero box duration is same as Exponential.

    """
    waning = None
    if box_duration == -1:
        waning = s2c.get_class_with_defaults( "WaningEffectConstant", schema_path )
    else:
        waning = s2c.get_class_with_defaults( "WaningEffectBoxExponential", schema_path )
        waning.Box_Duration = box_duration
        if decay_rate > 0: # -1 really
            waning.Decay_Time_Constant = 1.0/decay_rate
        else:
            waning.Decay_Time_Constant = 0

    waning.Initial_Effect = initial
    return waning


def _convert_prs( kvps ):
    """
    This function is not intended to be called directly by users but rather by emod_api.interventions.common
    API functions. It is intended to allow users to target interventions to individuals with specific 
    individual properties without having to worry about the particular data structures that the DTK uses.
    The permissible structures do need to be documented in the relevant API functions.
    kvps needs to be one of:
    - "key:value"
    - "key=value"
    - "key1:value1,key2:value2"
    - "key1=value1,key2=value2"
    - { "key": "value" }
    - { "key1": "value1", "key2": "value2" }
    - [  { "key1": "value1", "key2": "value2" }, { "key3": "value3", "key4": "value4" } ]
    - [ "key:value" ]
    - [ "key1:value1", "key2:value2" ]
    Note that the key-value-pairs _values_ are not currently validated against the selected demographics file.
    In the current implementation, the return value is prepped as a value for Property_Restrictions_Within_Nodes
    Args:
        kvps: any of the above
    Returns:
        list of dictionaries of key-value-pairs where each dict can only contain a given key once. E.g., [ { "Risk": "High" } ]
    """
    def parse_list_of_strings( entries ):
        ret_kvps = []
        prs_as_dict = dict()
        for entry in entries:
            if ":" in entry:
                key, value = entry.split(':')
                prs_as_dict[key.strip()] = value.strip()
            elif "=" in entry:
                key, value = entry.split('=')
                prs_as_dict[key.strip()] = value.strip()
            else:
                raise ValueError( f"ERROR: malformed property targets: {kvps}. Can't figure out where the keys and values are in at least one of the list entries." )
        ret_kvps = [ prs_as_dict ]
        return ret_kvps 

    ret_kvps = []
    if kvps is None or kvps == "":
        ret_kvps = list() # just happen to know default is empty list, not getting from schema
    elif type(kvps) is str and len(kvps)>0:
        entries = kvps.split(',')
        ret_kvps = parse_list_of_strings( entries )
    elif type(kvps) is dict:
        ret_kvps = [kvps]
    elif type(kvps) is list:
        if len(kvps)>0:
            if type(kvps[0]) in [ dict, set ]:
                ret_kvps = kvps # could check the contents of the list...
            elif type(kvps[0]) is str:
                ret_kvps = parse_list_of_strings( kvps )
            else:
                raise ValueError( f"ERROR: malformed property targets: {kvps}. The input is a list but not of strings or dicts." )
    else:
        raise ValueError( f"ERROR: malformed property targets: {kvps}." )

    # TBD: Let's keep simple things simple and leave complex solutions for complex cases.
    # Do not return list of dicts unless complex case.
    if len(ret_kvps) == 1:
        string_list = []
        for key, value in dict(ret_kvps[0]).items():
            string_list.append( f"{key}:{value}" )
        ret_kvps = string_list
    return ret_kvps
